@@ -126,8 +126,10 @@ def _add_cdrom_synaptic(mount_path):
 def add_cdrom(mount_path):
     if os.path.exists("/usr/bin/synaptic-pkexec"):
         _add_cdrom_synaptic(mount_path)
-    else:
+    elif os.path.exists("/usr/bin/software-properties-gtk"):
         _add_cdrom_sp(mount_path)
+    else:
+        logging.error("neither synaptic nor software-properties-gtk are installed to add CDROMs")
 
 
 if __name__ == "__main__":
