@@ -26,6 +26,8 @@ import string
 import debconf
 from datetime import datetime
 
+import apt_pkg
+
 # avoid hanging forever (LP: #1243090)
 import socket
 socket.setdefaulttimeout(60)
@@ -143,6 +145,10 @@ def trigger_update_notifier(failures, permanent=False):
                 input = f.read()
             output_file = open(NOTIFIER_FILE, 'w', encoding='utf-8')
     except Exception:
+        try:
+            output_file.close()
+        except Exception:
+            pass
         # Things failed and we can't even notify about it.  Break the
         # trigger so that there's some error propagation, even if not
         # the most pleasant sort.
@@ -160,8 +166,8 @@ def get_hook_file_names():
     res = []
     for relfile in os.listdir(DATADIR):
         # ignore files ending in .dpkg-*
-        if (os.path.splitext(relfile)[1] and
-                os.path.splitext(relfile)[1].startswith(".dpkg")):
+        if (os.path.splitext(relfile)[1]
+                and os.path.splitext(relfile)[1].startswith(".dpkg")):
             continue
         res.append(relfile)
     return res
@@ -176,6 +182,18 @@ def download_file(uri, sha256_hashsum):
     """
     download_dir = os.path.join(STAMPDIR, "partial")
     dest_file = os.path.join(download_dir, os.path.basename(uri))
+    try:
+        with open(dest_file, 'rb') as dest_file_obj:
+            # apt_pkg can directly hash from file obj, let's use that instead
+            # of hashlib
+            real_sha256 = apt_pkg.sha256sum(dest_file_obj)
+            if real_sha256 == sha256_hashsum:
+                return dest_file
+            else:
+                os.remove(dest_file)
+    except FileNotFoundError:
+        pass
+
     ret = subprocess.call(
         ["/usr/lib/apt/apt-helper",
          "download-file", uri, dest_file, "SHA256:" + sha256_hashsum])
@@ -233,70 +251,71 @@ def process_download_requests():
         hook = debian.deb822.Deb822()
         files = []
         sums = []
-        for para in hook.iter_paragraphs(open(file)):
-            if 'Script' in para:
-                if not files:
+        with open(file) as fobj:
+            for para in hook.iter_paragraphs(fobj):
+                if 'Script' in para:
+                    if not files:
+                        record_failure(relfile)
+                        break
+                    command = [para['Script']]
+
+                    if 'Should-Download' in para:
+                        db = debconf.DebconfCommunicator('update-notifier')
+                        try:
+                            should = db.get(para['Should-Download'])
+                            if should == "false":
+                                # Do nothing with this file.
+                                break
+                        except (DebconfError, KeyError):
+                            pass
+                        finally:
+                            db.shutdown()
+
+                    print_maybe("%s: processing..." % (relfile))
+
+                    # Download each file and verify the sum
+                    try:
+                        downloaded = set()
+                        for i in range(len(files)):
+                            print_maybe("%s: downloading %s" % (relfile, files[i]))
+                            dest_file = download_file(files[i], sums[i])
+                            if dest_file:
+                                command.append(dest_file)
+                                downloaded.add(dest_file)
+                            else:
+                                record_failure(relfile)
+                                break
+                        if relfile in failures + permanent_failures:
+                            break
+
+                        sys.stdout.flush()
+                        result = subprocess.call(command)
+                        if result:
+                            # There's no sense redownloading if the script fails
+                            permanent_failures.append(relfile)
+                        else:
+                            create_or_update_stampfile(stampfile)
+                        # cleanup
+                        for f in downloaded:
+                            os.remove(f)
+                        break
+                    except Exception:
+                        traceback.print_exc(file=sys.stderr)
+
+                    record_failure(relfile)
+                    # The 'script' is always the last stanza
+                    break
+
+                # Not in a 'script' stanza, so we should have some urls
+                try:
+                    files.append(para['Url'])
+                    sums.append(para['Sha256'])
+                except Exception as e:
+                    print_maybe("%s: Error processing!" % (relfile))
+                    if not isinstance(e, KeyError):
+                        traceback.print_exc(file=sys.stderr)
                     record_failure(relfile)
                     break
-                command = [para['Script']]
-
-                if 'Should-Download' in para:
-                    db = debconf.DebconfCommunicator('update-notifier')
-                    try:
-                        should = db.get(para['Should-Download'])
-                        if should == "false":
-                            # Do nothing with this file.
-                            break
-                    except (DebconfError, KeyError):
-                        pass
-                    finally:
-                        db.shutdown()
-
-                print_maybe("%s: processing..." % (relfile))
-
-                # Download each file and verify the sum
-                try:
-                    downloaded = set()
-                    for i in range(len(files)):
-                        print_maybe("%s: downloading %s" % (relfile, files[i]))
-                        dest_file = download_file(files[i], sums[i])
-                        if dest_file:
-                            command.append(dest_file)
-                            downloaded.add(dest_file)
-                        else:
-                            record_failure(relfile)
-                            break
-                    if relfile in failures + permanent_failures:
-                        break
-
-                    sys.stdout.flush()
-                    result = subprocess.call(command)
-                    if result:
-                        # There's no sense redownloading if the script fails
-                        permanent_failures.append(relfile)
-                    else:
-                        create_or_update_stampfile(stampfile)
-                    # cleanup
-                    for f in downloaded:
-                        os.remove(f)
-                    break
-                except Exception:
-                    traceback.print_exc(file=sys.stderr)
-
-                record_failure(relfile)
-                # The 'script' is always the last stanza
-                break
-
-            # Not in a 'script' stanza, so we should have some urls
-            try:
-                files.append(para['Url'])
-                sums.append(para['Sha256'])
-            except Exception as e:
-                print_maybe("%s: Error processing!" % (relfile))
-                if not isinstance(e, KeyError):
-                    traceback.print_exc(file=sys.stderr)
-                record_failure(relfile)
-                break
 
     previous_failures = existing_permanent_failures()
 
