@@ -3,6 +3,7 @@
 import hashlib
 import http.server as SimpleHTTPServer
 import os
+import pathlib
 import random
 import shutil
 import socketserver
@@ -13,6 +14,8 @@ import unittest
 
 from mock import patch
 from multiprocessing import Process
+import multiprocessing
+multiprocessing.set_start_method('fork')
 
 sys.path.insert(0, "data")
 sys.path.insert(1, "../data")
@@ -62,7 +65,7 @@ class DownloadTestCase(unittest.TestCase):
             uri, self.canary_hashsum)
         self.assertNotEqual(destfile, None)
         self.assertEqual(hashlib.sha256(
-            open(destfile).read().encode('utf-8')).hexdigest(),
+            pathlib.Path(destfile).read_text().encode('utf-8')).hexdigest(),
             self.canary_hashsum)
 
 
@@ -113,17 +116,27 @@ class ProcessDownloadRequestsTestCase(unittest.TestCase):
             foo.write("\n")
             foo.write("Script: %s" % script)
 
+    def _backdate(self, file, *, before):
+        """Backdate `file` to 1 second before `before`"""
+        if os.path.relpath(before):
+            before = os.path.join(package_data_downloader.DATADIR, before)
+        if os.path.relpath(file):
+            file = os.path.join(package_data_downloader.DATADIR, file)
+        before_st = os.stat(before)
+        sec_ns = 10**9
+        os.utime(file, ns=(before_st.st_atime_ns - sec_ns, before_st.st_mtime_ns - sec_ns))
+
     def test_hookfile_older_than_stampfile(self):
         # hook file older than stamp file nothing happens
         # create the hook file
         hookfile = "older-hookfile"
         self._setup_hook_file(hookfile)
-        time.sleep(0.01)
         # create the stamp file
         stampfile = os.path.join(package_data_downloader.STAMPDIR,
                                  hookfile)
         with open(stampfile, "w"):
-                pass
+            pass
+        self._backdate(hookfile, before=stampfile)
         orig_stamp_time = os.stat(stampfile).st_mtime
         package_data_downloader.process_download_requests()
         new_stamp_time = os.stat(stampfile).st_mtime
@@ -141,11 +154,12 @@ class ProcessDownloadRequestsTestCase(unittest.TestCase):
         stampfile = os.path.join(package_data_downloader.STAMPDIR,
                                  hookfile)
         with open(stampfile, "w"):
-                pass
+            pass
         orig_stamp_date = os.stat(stampfile).st_mtime
-        time.sleep(0.01)
         # create the hook file
         self._setup_hook_file(hookfile)
+        self._backdate(stampfile, before=hookfile)
+
         package_data_downloader.process_download_requests()
         # the download succeeded so the stamp file is touched
         new_stamp_date = os.stat(stampfile).st_mtime
@@ -164,18 +178,18 @@ class ProcessDownloadRequestsTestCase(unittest.TestCase):
         with open(stampfile, "w"):
             pass
         orig_stamp_date = os.stat(stampfile).st_mtime
-        time.sleep(0.01)
         # create the hook file
         self._setup_hook_file(hookfile)
+        self._backdate(hookfile, before=stampfile)
         fail_hookfile = "failure"
         # the stampfile indicates a failure
         fail_stampfile = os.path.join(package_data_downloader.STAMPDIR,
                                       "%s.failed" % fail_hookfile)
         with open(fail_stampfile, "w"):
             pass
-        time.sleep(0.01)
         # create the hook file
         self._setup_hook_file(fail_hookfile)
+        self._backdate(fail_hookfile, before=fail_stampfile)
         # create an empty notifier file
         with open(package_data_downloader.NOTIFIER_FILE, "w"):
             pass
@@ -190,10 +204,10 @@ class ProcessDownloadRequestsTestCase(unittest.TestCase):
         stampfile = os.path.join(package_data_downloader.STAMPDIR,
                                  hookfile)
         with open(stampfile, "w"):
-                pass
-        time.sleep(0.01)
+            pass
         # create the hook file
         self._setup_hook_file(hookfile, "/bin/false")
+        self._backdate(stampfile, before=hookfile)
         package_data_downloader.process_download_requests()
         # script failures are considered permanent
         self.assertTrue(os.path.exists(
@@ -206,9 +220,9 @@ class ProcessDownloadRequestsTestCase(unittest.TestCase):
         stampfile = os.path.join(package_data_downloader.STAMPDIR,
                                  hookfile)
         with open(stampfile, "w"):
-                pass
-        time.sleep(0.01)
+            pass
         self._setup_hook_file(hookfile)
+        self._backdate(stampfile, before=hookfile)
         # create an empty notifier file
         with open(package_data_downloader.NOTIFIER_FILE, "w"):
             pass
@@ -221,9 +235,9 @@ class ProcessDownloadRequestsTestCase(unittest.TestCase):
         stampfile = os.path.join(package_data_downloader.STAMPDIR,
                                  hookfile)
         with open(stampfile, "w"):
-                pass
-        time.sleep(0.01)
+            pass
         self._setup_hook_file(hookfile)
+        self._backdate(stampfile, before=hookfile)
         # create empty notifier files
         with open(package_data_downloader.NOTIFIER_FILE, "w"):
             pass
@@ -240,9 +254,9 @@ class ProcessDownloadRequestsTestCase(unittest.TestCase):
         stampfile = os.path.join(package_data_downloader.STAMPDIR,
                                  hookfile)
         with open(stampfile, "w"):
-                pass
-        time.sleep(0.01)
+            pass
         self._setup_hook_file(hookfile)
+        self._backdate(stampfile, before=hookfile)
         # create an empty permanent notifier file
         with open(package_data_downloader.NOTIFIER_PERMANENT_FILE, "w"):
             pass
@@ -256,12 +270,12 @@ class ProcessDownloadRequestsTestCase(unittest.TestCase):
         stampfile = os.path.join(package_data_downloader.STAMPDIR,
                                  hookfile)
         with open(stampfile, "w"):
-                pass
-        time.sleep(0.01)
+            pass
         # overwrite canary file to create a failure
         self.canary_file = "not-there.txt"
         # create the hook file
         self._setup_hook_file(hookfile)
+        self._backdate(stampfile, before=hookfile)
         package_data_downloader.process_download_requests()
         self.assertTrue(os.path.exists(
             os.path.join(package_data_downloader.STAMPDIR,
@@ -388,7 +402,7 @@ class PackageDataDownloaderTestCase(unittest.TestCase):
             "data/package-data-downloads-failed.in"
         package_data_downloader.trigger_update_notifier(
             failures=["foo", "bar"], permanent=False)
-        data = open(package_data_downloader.NOTIFIER_FILE).read()
+        data = pathlib.Path(package_data_downloader.NOTIFIER_FILE).read_text()
         self.assertEqual(data, """Priority: High
 _Name: Failure to download extra data files
 Terminal: True
